﻿using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Stack;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Interpreter.Expressions
{
    [TestClass]
    public class BooleanExpressionTests : CommonTests<bool>
    {
        protected override string TypeName { get { return "BOOLEAN"; } }
        protected override IStack<bool> Stack { get { return interpreter.BooleanStack; } }

        [TestMethod]
        public void TestAnd()
        {
            interpreter.BooleanStack.Push(true, false);
            interpreter.Interpret(new BooleanAndExpression());

            Assert.AreEqual(false, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestAndWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("AND", 1);
        }

        [TestMethod]
        public void TestOr()
        {
            interpreter.BooleanStack.Push(true, false);
            interpreter.Interpret(new BooleanOrExpression());

            Assert.AreEqual(true, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestOrWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("OR", 1);
        }

        [TestMethod]
        public void TestNot()
        {
            interpreter.BooleanStack.Push(true);
            interpreter.Interpret(new BooleanNotExpression());

            Assert.AreEqual(false, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestNotWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("NOT");
        }

        [TestMethod]
        public void TestFromFloat()
        {
            interpreter.FloatStack.Push(2.0);
            interpreter.Interpret(new BooleanFromFloatExpression());

            Assert.AreEqual(true, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestFromFloatWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("FROMFLOAT");
        }

        [TestMethod]
        public void TestFromInteger()
        {
            interpreter.IntegerStack.Push(2);
            interpreter.Interpret(new BooleanFromIntegerExpression());

            Assert.AreEqual(true, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestFromIntegerWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("FROMINTEGER");
        }

        protected override bool[] GetValues(int count)
        {
            var values = new bool[count];

            for (var i = 0; i < count; i++)
            {
                values[i] = i % 2 == 0;
            }

            return values;
        }

        protected override void CheckOtherStacksAreEmpty()
        {
            this.TestStackCounts(booleanStack: null);
        }
    }
}
